/* /////////////////////////////////////////////////////////////////////////
 * File:        xtests/xtests.h (formerly part of Synesis' internal test codebase)
 *
 * Purpose:     Main header file for xTests, a simple unit/component-testing
 *              library.
 *
 * Created:     20th June 1999
 * Updated:     23rd February 2008
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 1999-2008, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the
 *   names of any contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////// */


/** \file xtests/xtests.h
 *
 * [C, C++] Simple unit/component-testing library.
 */

#ifndef XTESTS_INCL_XTESTS_H_XTESTS
#define XTESTS_INCL_XTESTS_H_XTESTS

#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION
# define XTESTS_INCL_XTESTS_H_XTESTS_MAJOR      3
# define XTESTS_INCL_XTESTS_H_XTESTS_MINOR      5
# define XTESTS_INCL_XTESTS_H_XTESTS_REVISION   4
# define XTESTS_INCL_XTESTS_H_XTESTS_EDIT       231
#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////
 * Version information
 */

/**
 * \def _XTESTS_VER_MAJOR
 * The Major version number of the xTests library
 *
 * \def _XTESTS_VER_MINOR
 * Minor version number of the xTests library
 *
 * \def _XTESTS_VER_REVISION
 * The revision number of the xTests library
 *
 * \def _XTESTS_VER
 * The composite version of the xTests library
 */

#define _XTESTS_VER_MAJOR       (0)
#define _XTESTS_VER_MINOR       (1)
#define _XTESTS_VER_REVISION    (14)

#define _XTESTS_VER             (0x00010eff)

/* /////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <stlsoft/stlsoft.h>

#ifdef __cplusplus
# if !defined(_XTESTS_NO_CPP_API)
#  include <stlsoft/meta/is_integral_type.hpp>
#  include <stlsoft/meta/is_same_type.hpp>
#  include <stlsoft/meta/select_first_type_if.hpp>
# endif /* !_XTESTS_NO_CPP_API */
# if defined(STLSOFT_CF_EXCEPTION_SUPPORT)
#  include <new>
#  include <stdexcept>
# endif /* STLSOFT_CF_EXCEPTION_SUPPORT */
# if defined(STLSOFT_CF_RTTI_SUPPORT)
#  include <typeinfo>
# endif /* STLSOFT_CF_RTTI_SUPPORT */
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////
 * Compatibility
 */

#if _STLSOFT_VER < 0x010917ff
# error xTests requires version 1.9.23 (or later) of STLSoft; download from www.stlsoft.org
#endif /* _STLSOFT_VER */

/* /////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if defined(_STLSOFT_NO_NAMESPACE)
# define _XTESTS_NO_NAMESPACE
#endif /* _STLSOFT_NO_NAMESPACE */

#ifndef _XTESTS_NO_NAMESPACE
namespace xtests
{
namespace c
{
#endif /* !_XTESTS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////
 * Features
 */

#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION

# ifndef XTESTS_CALL
#  ifdef __cplusplus
#   define XTESTS_CALL(x)                   extern "C" x
#  else /* ? __cplusplus */
#   define XTESTS_CALL(x)                   extern x
#  endif /* __cplusplus */
#endif /* !XTESTS_CALL */

# ifdef STLSOFT_CF_FUNCTION_SYMBOL_SUPPORT
#  define XTESTS_GET_FUNCTION_()            __FUNCTION__
# else /* ? STLSOFT_CF_FUNCTION_SYMBOL_SUPPORT */
#  define XTESTS_GET_FUNCTION_()            stlsoft_static_cast(char const*, 0)
# endif /* STLSOFT_CF_FUNCTION_SYMBOL_SUPPORT */

# ifndef _XTESTS_NO_NAMESPACE
#  define XTESTS_NS_QUAL(ns, sym)           ns::sym
# else /* ? _XTESTS_NO_NAMESPACE */
#  define XTESTS_NS_QUAL(ns, sym)           sym
# endif /* _XTESTS_NO_NAMESPACE */

# define XTESTS_NS_C_QUAL(sym)              XTESTS_NS_QUAL(xtests::c, sym)
# define XTESTS_NS_CPP_QUAL(sym)            XTESTS_NS_QUAL(xtests::c::cpp, sym)

# ifdef STLSOFT_CF_RTTI_SUPPORT
#  define XTESTS_REPORT_EXCEPTION_(x)       XTESTS_NS_C_QUAL(xtests_caseExcepted)(typeid(x).name(), x.what())
# else /* ? STLSOFT_CF_RTTI_SUPPORT */
#  define XTESTS_REPORT_EXCEPTION_(x)       XTESTS_NS_C_QUAL(xtests_caseExcepted)("<rtti not available>", x.what())
# endif /* STLSOFT_CF_RTTI_SUPPORT */

# if defined(STLSOFT_CF_EXCEPTION_SUPPORT)

#  define XTESTS_INVOKE_TEST_CASE_FN_(fn, name)                                         \
                                                                                        \
    do                                                                                  \
    {                                                                                   \
        try                                                                             \
        {                                                                               \
            (*fn)();                                                                    \
        }                                                                               \
        catch(std::bad_alloc& x)                                                        \
        {                                                                               \
            XTESTS_REPORT_EXCEPTION_(x);                                                \
                                                                                        \
            throw;                                                                      \
        }                                                                               \
        catch(std::exception& x)                                                        \
        {                                                                               \
            XTESTS_REPORT_EXCEPTION_(x);                                                \
                                                                                        \
            /* XTESTS_CASE_END(name); */                                                      \
                                                                                        \
            /* throw; */                                                                      \
        }                                                                               \
    } while(0)

#  define XTESTS_INVOKE_TEST_CASE_FN_THROWS_(fn, name, type)                            \
                                                                                        \
    do                                                                                  \
    {                                                                                   \
        try                                                                             \
        {                                                                               \
            (*fn)();                                                                    \
                                                                                        \
            XTESTS_NS_C_QUAL(xtests_caseExceptionExpected)(#type);                      \
        }                                                                               \
        catch(type& /* x */)                                                            \
        {                                                                               \
        }                                                                               \
        catch(std::bad_alloc& x)                                                        \
        {                                                                               \
            XTESTS_REPORT_EXCEPTION_(x);                                                \
                                                                                        \
            throw;                                                                      \
        }                                                                               \
        catch(std::exception& x)                                                        \
        {                                                                               \
            XTESTS_REPORT_EXCEPTION_(x);                                                \
                                                                                        \
            /* XTESTS_CASE_END(name); */                                                      \
                                                                                        \
            /* throw; */                                                                      \
        }                                                                               \
    } while(0)

# else /* ? STLSOFT_CF_EXCEPTION_SUPPORT */

#  define XTESTS_INVOKE_TEST_CASE_FN_(fn, name)  (*fn)()

# endif /* STLSOFT_CF_EXCEPTION_SUPPORT */


#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////
 * Constants & definitions
 */

/** \def XTESTS_FP_APPROXIMATE_FACTOR
 * The factor within which floating point numbers are deemed to be
 * approximately equal.
 */
#define XTESTS_FP_APPROXIMATE_FACTOR                    (1.000001)

/* /////////////////////////////////////////////////////////////////////////
 * Macros
 */

/** \defgroup group__xtests__test_runner_functions Test Runner Functions
 *
 * Functions that are used to define, start, report on, and complete test
 * runners.
 */
 
/** \defgroup group__xtests__test_case_functions Test Case Functions
 *
 * Functions that are used to define, start, and complete test cases.
 */
 
/** \defgroup group__xtests__test_functions Test Functions
 *
 * Functions that are used to exercise tests.
 */
 
/** \defgroup group__xtests__utiliy_functions Utility Functions
 *
 * Utility functions that supplement use of the library.
 */
 

/** \def XTESTS_START_RUNNER(name, verbosity)
 *
 * \ingroup group__xtests__test_runner_functions
 *
 * Starts a test runner
 *
 * A test runner is a logically-related group of test cases.
 *
 * \param name The name of the test-runner
 * \param verbosity The verbosity (see xtests_verbosity_t) at which the
 *   runner will be executed
 */
#define XTESTS_START_RUNNER(name, verbosity)                                            \
                                                                                        \
    (0 == XTESTS_NS_C_QUAL(xtests_startRunner)((name), (verbosity), NULL, NULL))


/** \def XTESTS_START_RUNNER_WITH_REPORTER(name, verbosity, reporter, reporterParam)
 *
 * \ingroup group__xtests__test_runner_functions
 *
 * Starts a test runner with the given callback reporter
 *
 * A test runner is a logically-related group of test cases.
 *
 * \param name The name of the test-runner
 * \param verbosity The verbosity (see xtests_verbosity_t) at which the
 *   runner will be executed
 * \param reporter The reporter instance
 * \param reporterParam A caller-supplied parameter that is passed with
 *   every callback
 */
#define XTESTS_START_RUNNER_WITH_REPORTER(name, verbosity, reporter, reporterParam)     \
                                                                                        \
    (0 == XTESTS_NS_C_QUAL(xtests_startRunner)((name), (verbosity), (reporter), (reporterParam)))


/** \def XTESTS_PRINT_RESULTS()
 *
 * \ingroup group__xtests__test_runner_functions
 *
 * Prints the test results of the currently executing test.
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_START_RUNNER() or XTESTS_START_RUNNER_WITH_REPORTER(), and
 *   before invocation of XTESTS_END_RUNNER() or
 *   XTESTS_END_RUNNER_UPDATE_EXITCODE().
 */
#define XTESTS_PRINT_RESULTS()                                                          \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_printRunnerResults)()


/** \def XTESTS_END_RUNNER()
 *
 * \ingroup group__xtests__test_runner_functions
 *
 * Ends a test runner
 */
#define XTESTS_END_RUNNER()                                                             \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_endRunner)(NULL)


/** \def XTESTS_END_RUNNER_UPDATE_EXITCODE()
 *
 * \ingroup group__xtests__test_runner_functions
 *
 * Ends a test runner, and modifies a caller-supplied exit code parameter
 *
 * \param retCode A pointer to a variable of type <code>int</code> that will
 *   receive an exit code.
 *
 * \remarks The variable should have been initialised to
 *   <code>EXIT_SUCCESS</code>, and each invocation of
 *   XTESTS_END_RUNNER_UPDATE_EXITCODE() (for each separate test-runner
 *   in a given application) will only set it to <code>EXIT_FAILURE</code>
 *   in the case where that runner has failed one or more tests.
 */
#define XTESTS_END_RUNNER_UPDATE_EXITCODE(retCode)                                      \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_endRunner)(retCode)




/** \def XTESTS_CASE_BEGIN(name, desc)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Begins a test case, of the given name and description
 *
 * \param name The name of the test case
 * \param desc The description of the test case. May be <code>NULL</code>
 *   or the empty string (<code>""</code>).
 */
#define XTESTS_CASE_BEGIN(name, desc)                                                   \
                                                                                        \
    (0 == XTESTS_NS_C_QUAL(xtests_beginTestCase)((name), (desc)))


/** \def XTESTS_CASE_END(name, desc)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Ends the current test case
 *
 * \param name The name of the test case
 *
 * \note The <code>name</code> parameter is ignored in the current
 *   implementation, which can only run one test case at a time.
 */
#define XTESTS_CASE_END(name)                                                           \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_endTestCase)((name))


/** \def XTESTS_RUN_CASE_WITH_NAME_AND_DESC(name, desc, fn)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Runs the given test case function, specifying a name and description
 *
 * \param name Name of the test case
 * \param desc Description of the test case
 * \param fn A function, taking no parameters and returning
 *   <code>void</code>, that executes a number of tests representing a test
 *   case.
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_RUN_CASE_WITH_NAME_AND_DESC(name, desc, fn)                              \
                                                                                        \
    do                                                                                  \
    {                                                                                   \
        if(XTESTS_CASE_BEGIN(name, desc))                                               \
        {                                                                               \
            XTESTS_INVOKE_TEST_CASE_FN_(fn, name);                                      \
                                                                                        \
            XTESTS_CASE_END(name);                                                      \
        }                                                                               \
                                                                                        \
    } while(0)


/** \def XTESTS_RUN_CASE_WITH_DESC(fn, desc)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Runs the given test case function, specifying a description
 *
 * \param fn A function, taking no parameters and returning
 *   <code>void</code>, that executes a number of tests representing a test
 *   case.
 * \param desc Description of the test case
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_RUN_CASE_WITH_DESC(fn, desc)                                             \
                                                                                        \
    XTESTS_RUN_CASE_WITH_NAME_AND_DESC(#fn, desc, fn)
                                                                                        \
/** \def XTESTS_RUN_CASE(fn)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Runs the given test case function
 *
 * \param fn A function, taking no parameters and returning
 *   <code>void</code>, that executes a number of tests representing a test
 *   case.
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_RUN_CASE(fn)                                                             \
                                                                                        \
    XTESTS_RUN_CASE_WITH_DESC(fn, "")


#ifdef STLSOFT_CF_EXCEPTION_SUPPORT

/** \def XTESTS_RUN_CASE_THAT_THROWS_WITH_DESC(name, desc, fn, type)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Runs the given test case function, specifying a name and description, and
 * the type of an exception that is expected
 *
 * \param name Name of the test case
 * \param desc Description of the test case
 * \param fn A function, taking no parameters and returning
 *   <code>void</code>, that executes a number of tests representing a test
 *   case.
 * \param type The type of the exception that is expected to be thrown
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_RUN_CASE_THAT_THROWS_WITH_DESC(name, desc, fn, type)                    \
                                                                                        \
    do                                                                                  \
    {                                                                                   \
        if(XTESTS_CASE_BEGIN(name, desc))                                               \
        {                                                                               \
            XTESTS_INVOKE_TEST_CASE_FN_THROWS_(fn, name, type);                         \
                                                                                        \
            XTESTS_CASE_END(name);                                                      \
        }                                                                               \
                                                                                        \
    } while(0)

/** \def XTESTS_RUN_CASE_THAT_THROWS(fn, type)
 *
 * \ingroup group__xtests__test_case_functions
 *
 * Runs the given test case function, specifying the type of an exception
 * that is expected
 *
 * \param fn A function, taking no parameters and returning
 *   <code>void</code>, that executes a number of tests representing a test
 *   case.
 * \param type The type of the exception that is expected to be thrown
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_RUN_CASE_THAT_THROWS(fn, type)                                           \
                                                                                        \
    XTESTS_RUN_CASE_THAT_THROWS_WITH_DESC(#fn, "", fn, type)

#endif /* STLSOFT_CF_EXCEPTION_SUPPORT */


/** \def XTESTS_TEST_FAIL_WITH_QUALIFIER(msg, qualifier)
 *
 * \ingroup group__xtests__test_functions
 *
 * Causes a test failure to be expressed, passing an explanatory message and
 * a message qualifier.
 *
 * \param msg The message explaining the failure condition
 * \param qualifier The message qualifier.
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_FAIL_WITH_QUALIFIER(msg, qualifier)                                 \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_writeFailMessage)(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), msg, qualifier)

/** \def XTESTS_TEST_FAIL(msg)
 *
 * \ingroup group__xtests__test_functions
 *
 * Causes a test failure to be expressed, passing an explanatory message.
 *
 * \param msg The message explaining the failure condition
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_FAIL(msg)                                                           \
                                                                                        \
    XTESTS_FAIL_WITH_QUALIFIER(msg, NULL)

/** \def XTESTS_TEST_PASSED()
 *
 * \ingroup group__xtests__test_functions
 *
 * Causes a test success to be expressed.
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_PASSED()                                                            \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_testPassed)()

/** \def XTESTS_TEST_WITH_MESSAGE(expr, msg)
 *
 * \ingroup group__xtests__test_functions
 *
 * Causes a test to be exercised, passing an explanatory message to be used
 * in the case of failure.
 *
 * \param expr The expression whose truth is to be evaluated
 * \param msg The message explaining the failure condition
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_WITH_MESSAGE(expr, msg)                                             \
                                                                                        \
    ((expr)                                                                             \
        ? XTESTS_NS_C_QUAL(xtests_testPassed)()                                         \
        : XTESTS_NS_C_QUAL(xtests_testFailed)(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), msg))

/** \def XTESTS_TEST(expr)
 *
 * \ingroup group__xtests__test_functions
 *
 * Causes a test to be exercised.
 *
 * \param expr The expression whose truth is to be evaluated
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST(expr)                                                               \
                                                                                        \
    XTESTS_TEST_WITH_MESSAGE(expr, #expr)

#if defined(__cplusplus) && \
    !defined(_XTESTS_NO_CPP_API)

/** \def XTESTS_TEST_INTEGER_EQUAL_EXACT(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two integer values are exactly equal.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \remarks The integers should be of the same type
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_INTEGER_EQUAL_EXACT(expected, actual)                              \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_integer(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonEqual)))

/** \def XTESTS_TEST_INTEGER_NOT_EQUAL(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two integer values are not equal.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \remarks The integers should be of the same type
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_INTEGER_NOT_EQUAL(expected, actual)                                \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_integer(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonNotEqual)))

/** \def XTESTS_TEST_INTEGER_GREATER(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that the actual integer value is greater than the expected value.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_INTEGER_GREATER(expected, actual)                                  \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_integer(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonGreaterThan)))

/** \def XTESTS_TEST_INTEGER_LESS(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that the actual integer value is less than the expected value.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_INTEGER_LESS(expected, actual)                                     \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_integer(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonLessThan)))

/** \def XTESTS_TEST_INTEGER_GREATER_OR_EQUAL(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that the actual integer value is greater than or equal to the
 * expected value.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_INTEGER_GREATER_OR_EQUAL(expected, actual)                         \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_integer(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonGreaterThanOrEqual)))

/** \def XTESTS_TEST_INTEGER_LESS_OR_EQUAL(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that the actual integer value is less than or equal to the expected
 * value.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_INTEGER_LESS_OR_EQUAL(expected, actual)                            \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_integer(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonLessThanOrEqual)))


/** \def XTESTS_TEST_FLOATINGPOINT_EQUAL_EXACT(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two floating point values are exactly equal
 *
 * \param expected The expected floating point value
 * \param actual The actual floating point value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_FLOATINGPOINT_EQUAL_EXACT(expected, actual)                        \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_floating_point(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonEqual)))

/** \def XTESTS_TEST_FLOATINGPOINT_EQUAL_APPROX(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two floating point values are approximately
 * equal (as defined by XTESTS_FP_APPROXIMATE_FACTOR).
 *
 * \param expected The expected floating point value
 * \param actual The actual floating point value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
# define XTESTS_TEST_FLOATINGPOINT_EQUAL_APPROX(expected, actual)                       \
                                                                                        \
    XTESTS_NS_CPP_QUAL(xtests_test_floating_point(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonApproxEqual)))

#else /* ? __cplusplus) && !_XTESTS_NO_CPP_API */

# define XTESTS_TEST_INTEGER_EQUAL_EXACT(expected, actual)          XTESTS_TEST((expected) == (actual))
# define XTESTS_TEST_INTEGER_NOT_EQUAL(expected, actual)            XTESTS_TEST((expected) != (actual))
# define XTESTS_TEST_INTEGER_GREATER(expected, actual)              XTESTS_TEST((expected) < (actual))
# define XTESTS_TEST_INTEGER_LESS(expected, actual)                 XTESTS_TEST((expected) > (actual))
# define XTESTS_TEST_INTEGER_GREATER_OR_EQUAL(expected, actual)     XTESTS_TEST((expected) <= (actual))
# define XTESTS_TEST_INTEGER_LESS_OR_EQUAL(expected, actual)        XTESTS_TEST((expected) >= (actual))

# define XTESTS_TEST_FLOATINGPOINT_EQUAL_APPROX(expected, actual)   XTESTS_TEST(xtests_floatingPointClose((expected), (actual)))

#endif /* __cplusplus) && !_XTESTS_NO_CPP_API */


/** \def XTESTS_TEST_INTEGER_EQUAL(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two integer values (of the same type) are equal.
 *
 * \param expected The expected integer value
 * \param actual The actual integer value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_INTEGER_EQUAL(expected, actual)                 XTESTS_TEST_INTEGER_EQUAL_EXACT(expected, actual)

/** \def XTESTS_TEST_FLOATINGPOINT_EQUAL(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two floating point values (of the same type) are equal.
 *
 * \param expected The expected floating point value
 * \param actual The actual floating point value
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_FLOATINGPOINT_EQUAL(expected, actual)           XTESTS_TEST_FLOATINGPOINT_EQUAL_APPROX(expected, actual)



/** \def XTESTS_TEST_MULTIBYTE_STRINGS_EQUAL(expected, actual)
 *
 * \ingroup group__xtests__test_functions
 *
 * Tests that two (multibyte) strings are equal.
 *
 * \param expected The expected value of the string
 * \param actual The actual value of the string
 *
 * \note This can only be invoked after a successful invocation of
 *   XTESTS_CASE_BEGIN() and before invocation of XTESTS_CASE_END().
 */
#define XTESTS_TEST_MULTIBYTE_STRINGS_EQUAL(expected, actual)                           \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_testMultibyteStrings(__FILE__, __LINE__, XTESTS_GET_FUNCTION_(), (expected), (actual), XTESTS_NS_C_QUAL(xtestsComparisonEqual)))


/* /////////////////////////////////////////////////////////////////////////
 * API functions
 */

/** The verbosity level to be applied when executing a test runner
 *
 * \ingroup group__xtests__test_runner_functions
 *
 * \see XTESTS_START_RUNNER()
 * \see XTESTS_START_RUNNER_WITH_REPORTER()
 */
enum xtests_verbosity_t
{
        xtestsVerbositySilent           =   0   /*!< Don't say anything at any point */
    ,   xtestsVerbositySummaryOnError   =   1   /*!< Show a summary, but only on error */
    ,   xtestsVerbosityVerboseOnError   =   2   /*!< Give full information, but only on error */
    ,   xtestsVerbositySummaryOnSuccess =   3   /*!< Give full information on error, otherwise just summary */
    ,   xtestsVerbosityVerboseOnSuccess =   4   /*!< Give full information at all times */

};
#ifndef __cplusplus
typedef enum xtests_verbosity_t xtests_verbosity_t;
#endif /* !__cplusplus */


#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION

enum xtests_comparison_t
{
        xtestsComparisonEqual               =   0
    ,   xtestsComparisonNotEqual
    ,   xtestsComparisonGreaterThan
    ,   xtestsComparisonLessThan
    ,   xtestsComparisonGreaterThanOrEqual
    ,   xtestsComparisonLessThanOrEqual
    ,   xtestsComparisonApproxEqual
    ,   xtestsComparison_max_enumerator

};
# ifndef __cplusplus
typedef enum xtests_comparison_t xtests_comparison_t;
# endif /* !__cplusplus */


# ifdef __cplusplus
struct xTests_runner_results_t
{
    typedef stlsoft_ns_qual(uint32_t)   uint32_t;

    char const*     name;
    uint32_t        numCases;
    uint32_t        numTests;
    uint32_t        numTestsSucceeded;
    uint32_t        numExpectedExceptions;
    uint32_t        numUnexpectedExceptions;
};

struct xTests_Reporter_t
{
protected:
#  if defined(STLSOFT_COMPILER_IS_GCC)
    virtual ~xTests_Reporter_t()
    {}
#  else /* ? compiler */
    ~xTests_Reporter_t()
    {}
private:
    virtual void dummy_dtor()
    {}
#  endif /* compiler */

public: /* Overrides */
    virtual void onStartRunner(void* reporterParam, char const* name, int verbosity) = 0;

      virtual void onBeginTestCase(void* reporterParam, char const* name, char const* desc, int verbosity) = 0;

        virtual void onTestPassed(void* reporterParam, int verbosity) = 0;

        virtual void onTestFailed(void* reporterParam, char const* file, int line, char const* function, char const* expr, int verbosity) = 0;

        virtual void onCaseExcepted(void* reporterParam, char const* caseName, char const* exceptionType, char const* exceptionMessage, int verbosity) = 0;

        virtual void onCaseExceptionExpected(void* reporterParam, char const* caseName, char const* exceptionType, int verbosity) = 0;

      virtual void onEndTestCase(void* reporterParam, char const* name, xTests_runner_results_t const* results, int verbosity) = 0;

      virtual void onPrintRunnerResults(void* reporterParam, xTests_runner_results_t const* results, int verbosity) = 0;

      virtual void onAbend(void* reporterParam, char const* message, int verbosity) = 0;

      virtual void onDefect(void* reporterParam, char const* message, char const* qualifier, int verbosity) = 0;

    virtual void onEndRunner(void* reporterParam, char const* name, int verbosity) = 0;
};
# else /* ? __cplusplus */
struct xTests_Reporter_t;
typedef struct xTests_Reporter_t xTests_Reporter_t;
# endif /* __cplusplus */


#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */


#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION

XTESTS_CALL(int) xtests_startRunner(char const* name, int verbosity, xTests_Reporter_t* reporter /* = NULL */, void* reporterParam /* = NULL */);

XTESTS_CALL(int) xtests_endRunner(int *retCode);


XTESTS_CALL(void) xtests_printRunnerResults(void);


XTESTS_CALL(void) xtests_abend(char const* message);


XTESTS_CALL(int) xtests_beginTestCase(char const* name, char const* desc);

XTESTS_CALL(int) xtests_endTestCase(char const* name);


XTESTS_CALL(int) xtests_testPassed(void);

XTESTS_CALL(int) xtests_testFailed( char const* file
                                ,   int         line
                                ,   char const* function
                                ,   char const* expr);

XTESTS_CALL(int) xtests_testFailed_int(     char const*                     file
                                        ,   int                             line
                                        ,   char const*                     function
                                        ,   int                             expected
                                        ,   int                             actual
                                        ,   xtests_comparison_t             comp);
XTESTS_CALL(int) xtests_testFailed_long(    char const*                     file
                                        ,   int                             line
                                        ,   char const*                     function
                                        ,   long                            expected
                                        ,   long                            actual
                                        ,   xtests_comparison_t             comp);
XTESTS_CALL(int) xtests_testFailed_ulong(   char const*                     file
                                        ,   int                             line
                                        ,   char const*                     function
                                        ,   unsigned long                   expected
                                        ,   unsigned long                   actual
                                        ,   xtests_comparison_t             comp);
# ifdef STLSOFT_CF_64BIT_INT_SUPPORT
XTESTS_CALL(int) xtests_testFailed_longlong(char const*                     file
                                        ,   int                             line
                                        ,   char const*                     function
                                        ,   stlsoft_ns_qual(ss_sint64_t)    expected
                                        ,   stlsoft_ns_qual(ss_sint64_t)    actual
                                        ,   xtests_comparison_t             comp);
# endif /* STLSOFT_CF_64BIT_INT_SUPPORT */
XTESTS_CALL(int) xtests_testFailed_double(  char const*                     file
                                        ,   int                             line
                                        ,   char const*                     function
                                        ,   double                          expected
                                        ,   double                          actual
                                        ,   xtests_comparison_t             comp);

XTESTS_CALL(int) xtests_testMultibyteStrings(   char const*         file
                                            ,   int                 line
                                            ,   char const*         function
                                            ,   char const*         expected
                                            ,   char const*         actual
                                            ,   xtests_comparison_t comp);

XTESTS_CALL(int) xtests_writeFailMessage(char const* file
                                    ,   int         line
                                    ,   char const* function
                                    ,   char const* message
                                    ,   char const* qualifyingInformation);


XTESTS_CALL(void) xtests_caseExcepted(char const* exceptionType, char const* exceptionMessage);

XTESTS_CALL(void) xtests_caseExceptionExpected(char const* exceptionType);


XTESTS_CALL(int) xtests_floatingPointClose(double expected, double actual);

XTESTS_CALL(double) xtests_setFloatingPointCloseFactor(double factor, double *old /* = NULL */);

#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////
 * C++-only functionality
 */

#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION

# if defined(__cplusplus) && \
     !defined(_XTESTS_NO_CPP_API)

#  ifndef _XTESTS_NO_NAMESPACE
namespace cpp
{
#  endif /* !_XTESTS_NO_NAMESPACE */


template <typename T>
struct xtests_failure_reporter;

template <>
struct xtests_failure_reporter<int>
{
    static void xtests_report_failure_equal(char const* file, int line, char const* function, int expected, int actual, xtests_comparison_t comp)
    {
        xtests_testFailed_int(file, line, function, expected, actual, comp);
    }
};

template <>
struct xtests_failure_reporter<long>
{
    static void xtests_report_failure_equal(char const* file, int line, char const* function, long expected, long actual, xtests_comparison_t comp)
    {
        xtests_testFailed_long(file, line, function, expected, actual, comp);
    }
};

template <>
struct xtests_failure_reporter<size_t>
{
    static void xtests_report_failure_equal(char const* file, int line, char const* function, size_t expected, size_t actual, xtests_comparison_t comp)
    {
        xtests_testFailed_ulong(file, line, function, expected, actual, comp);
    }
};

#  ifdef STLSOFT_CF_64BIT_INT_SUPPORT
template <>
struct xtests_failure_reporter<stlsoft::sint64_t>
{
    static void xtests_report_failure_equal(char const* file, int line, char const* function, stlsoft::sint64_t expected, stlsoft::sint64_t actual, xtests_comparison_t comp)
    {
        xtests_testFailed_longlong(file, line, function, expected, actual, comp);
    }
};
#  endif /* STLSOFT_CF_64BIT_INT_SUPPORT */

template<   typename T1
        ,   typename T2
        >
struct xtests_integer_failure_reporter_selector
{
private:
    enum { are_types_same       =   (0 != stlsoft::is_same_type<T1, T2>::value) };
    enum { T1_is_larger_than_T2 =   sizeof(T1) > sizeof(T2)                     };

    typedef typename stlsoft::select_first_type_if< T1
                                                ,   T2
                                                ,   T1_is_larger_than_T2
                                                >::type         larger_type_;

public:
    typedef xtests_failure_reporter<larger_type_>               type;
};

template <typename I1, typename I2>
inline void xtests_reportFailedIntegerComparison(char const* file, int line, char const* function, I1 const& expected, I2 const& actual, xtests_comparison_t comp)
{
    STLSOFT_STATIC_ASSERT(0 != stlsoft::is_integral_type<I1>::value);
    STLSOFT_STATIC_ASSERT(0 != stlsoft::is_integral_type<I2>::value);

#  if defined(STLSOFT_COMPILER_IS_BORLAND)

    xtests_integer_failure_reporter_selector<I1, I2>::type::xtests_report_failure_equal(file, line, function, expected, actual, comp);

#  else /* ? compiler */

    typedef typename xtests_integer_failure_reporter_selector<I1, I2>::type    failure_reporter_t;

    failure_reporter_t::xtests_report_failure_equal(file, line, function, expected, actual, comp);

#  endif /* compiler */
}

inline void xtests_reportFailedFloatingPointComparison(char const* file, int line, char const* function, double const& expected, double const& actual, xtests_comparison_t comp)
{
    xtests_testFailed_double(file, line, function, expected, actual, comp);
}

template <typename I1, typename I2>
inline int xtests_test_integer(char const* file, int line, char const* function, I1 const& expected, I2 const& actual, xtests_comparison_t comp)
{
    int comparisonSucceeded = false;

    switch(comp)
    {
        case    xtestsComparisonEqual:
        case    xtestsComparisonApproxEqual:
            if(expected == actual)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonNotEqual:
            if(expected != actual)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonGreaterThan:
            if(actual > expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonLessThan:
            if(actual < expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonGreaterThanOrEqual:
            if(actual >= expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonLessThanOrEqual:
            if(actual <= expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparison_max_enumerator:
            xtests_abend("invalid test comparison type: test framework may be out of date!");
            break;
    }

    if(comparisonSucceeded)
    {
        xtests_testPassed();
    }
    else
    {
        xtests_reportFailedIntegerComparison(file, line, function, expected, actual, comp);
    }

    return comparisonSucceeded;
}

inline int xtests_test_floating_point(char const* file, int line, char const* function, double const& expected, double const& actual, xtests_comparison_t comp)
{
    int comparisonSucceeded = false;

    switch(comp)
    {
        case    xtestsComparisonEqual:
            if(expected == actual)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonApproxEqual:
            if(xtests_floatingPointClose(expected, actual))
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonNotEqual:
            if(expected != actual)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonGreaterThan:
            if(actual > expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonLessThan:
            if(actual < expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonGreaterThanOrEqual:
            if(actual >= expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparisonLessThanOrEqual:
            if(actual <= expected)
            {
                comparisonSucceeded = true;
            }
            break;
        case    xtestsComparison_max_enumerator:
            xtests_abend("invalid test comparison type: test framework may be out of date!");
            break;
    }

    if(comparisonSucceeded)
    {
        xtests_testPassed();
    }
    else
    {
        xtests_reportFailedFloatingPointComparison(file, line, function, expected, actual, comp);
    }

    return comparisonSucceeded;
}

#  ifndef _XTESTS_NO_NAMESPACE
} /* namespace cpp */
#  endif /* !_XTESTS_NO_NAMESPACE */

# endif /* __cplusplus) && !_XTESTS_NO_CPP_API */

#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION
XTESTS_CALL(int) xtests_commandLine_parseVerbosity(int argc, char** argv, int *verbosity);
#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */

/** \def XTESTS_COMMANDLINE_PARSEVERBOSITY(argc, argv, pverbosity)
 *
 * \ingroup group__xtests__utiliy_functions
 *
 * Parses a verbosity from the command-line
 *
 * Parse the verbosity from the command-line arguments, looking for an
 * argument of the form "--verbosity=<N>", where N is a non-negative
 * integer.
 *
 * \param argc The <code>argc</code> parameter passed into
 *   <code>main()</code>
 * \param argv The <code>argv</code> parameter passed into
 *   <code>main()</code>
 * \param pverbosity A pointer to an integer to receive the verbosity. Will
 *   be set to xtestsVerbositySummaryOnSuccess even if no verbosity argument
 *   is found. May not be NULL.
 *
 * \return The index of argument containing the verbosity, or 0 to indicate
 *   failure
 */
#define XTESTS_COMMANDLINE_PARSEVERBOSITY(argc, argv, pverbosity)                       \
                                                                                        \
    XTESTS_NS_C_QUAL(xtests_commandLine_parseVerbosity)((argc), (argv), (pverbosity))

/* /////////////////////////////////////////////////////////////////////////
 * Obsolete names
 */

#ifndef XTESTS_DOCUMENTATION_SKIP_SECTION
# define XTESTS_FAIL_WITH_QUALIFIER(msg, qualifier)     XTESTS_TEST_FAIL_WITH_QUALIFIER(msg, qualifier)
# define XTESTS_FAIL(msg)                               XTESTS_TEST_FAIL(msg)
# define XTESTS_PASSED()                                XTESTS_TEST_PASSED()
#endif /* !XTESTS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _XTESTS_NO_NAMESPACE
} /* namespace c */
} /* namespace xtests */
#endif /* !_XTESTS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////// */

#endif /* !XTESTS_INCL_XTESTS_H_XTESTS */

/* ////////////////////////////////////////////////////////////////////// */
