/* /////////////////////////////////////////////////////////////////////////////
 * File:    uio.c
 *
 * Purpose: Vector file read/write.
 *
 * Created: 19th September 2005
 * Updated: 17th February 2008
 *
 * Home:    http://synesis.com.au/software/
 *
 * Copyright (c) 2005-2008, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the names of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef UNIXEM_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_C_UIO_MAJOR       1
# define _SYNSOFT_VER_C_UIO_MINOR       0
# define _SYNSOFT_VER_C_UIO_REVISION    6
# define _SYNSOFT_VER_C_UIO_EDIT        7
#endif /* !UNIXEM_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <sys/uio.h>

#include <unixem/unixem.h>
#include <unixem/internal/util.h>

#include <errno.h>
#include <windows.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Feature support
 */

#if defined(__BORLANDC__)
UNIXEM_STGCLS_IMP long _cdecl _get_osfhandle(int __handle);
#elif defined(__DMC__) || \
      defined(__INTEL_COMPILER) || \
      defined(__MWERKS__) || \
      defined(_MSC_VER)
UNIXEM_STGCLS_IMP long __cdecl _get_osfhandle(int __handle);
#elif defined(__GNUC__)
long __cdecl _get_osfhandle(int __handle);
#elif defined(__WATCOMC__)
_WCRTLINK extern long _get_osfhandle( int __posixhandle );
#else
# error Compiler not discriminated
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

ssize_t readv(int fd, struct iovec const *vector, int count)
{
    int     i;
    size_t  total;
    void    *pv;
    int     ret;

    /* Determine the total size. */
    for(i = 0, total = 0; i < count; ++i)
    {
        total += vector[i].iov_len;
    }

    pv = HeapAlloc(GetProcessHeap(), 0, total);

    if(NULL == pv)
    {
        errno = errno_from_Win32(GetLastError());

        ret = -1;
    }
    else
    {
        HANDLE  h   =   (HANDLE)_get_osfhandle(fd);
        DWORD   dw;

        if(!ReadFile(h, pv, total, &dw, NULL))
        {
            errno = errno_from_Win32(GetLastError());

            ret = -1;
        }
        else
        {
            for(i = 0, ret = 0; i < count && 0 != dw; ++i)
            {
                size_t  n   =   (dw < vector[i].iov_len) ? dw : vector[i].iov_len;

                (void)memcpy(vector[i].iov_base, (char const*)pv + ret, n);

                ret +=  n;
                dw  -=  n;
            }
        }

        (void)HeapFree(GetProcessHeap(), 0, pv);
    }

    return ret;
}

ssize_t writev(int fd, struct iovec const *vector, int count)
{
    int     i;
    size_t  total;
    void    *pv;
    int     ret;

    /* Determine the total size. */
    for(i = 0, total = 0; i < count; ++i)
    {
        total += vector[i].iov_len;
    }

    pv = HeapAlloc(GetProcessHeap(), 0, total);

    if(NULL == pv)
    {
        errno = errno_from_Win32(GetLastError());

        ret = -1;
    }
    else
    {
        HANDLE  h   =   (HANDLE)_get_osfhandle(fd);
        DWORD   dw;

        for(i = 0, ret = 0; i < count; ++i)
        {
            (void)memcpy((char*)pv + ret, vector[i].iov_base, vector[i].iov_len);

            ret +=  vector[i].iov_len;
        }

        if(!WriteFile(h, pv, total, &dw, NULL))
        {
            errno = errno_from_Win32(GetLastError());

            ret = -1;
        }
        else
        {
            ret = (int)dw;
        }

        (void)HeapFree(GetProcessHeap(), 0, pv);
    }

    return ret;
}

/* ////////////////////////////////////////////////////////////////////////// */
